@@ -1,8 +1,10 @@
 /*
 
   Copyright (C) 2003  Razvan Cojocaru <razvanco@gmx.net>
-   XML-RPC/Context ID code contributed by Eamon Millman / PCI Geomatics
-  <millman@pcigeomatics.com>
+  XML-RPC/Context ID code contributed by Eamon Millman / PCI Geomatics
+    <millman@pcigeomatics.com>
+  Bugfixes contributed by Kuang-che Wu 
+    https://sourceforge.net/users/kcwu/
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -37,7 +39,6 @@
 #include <hhcparser.h>
 
 
-
 // damn wxWidgets and it's scoped ptr.
 class UCharPtr {
 public:
@@ -226,7 +227,7 @@ void CHMFile::CloseCHM()
 #define EMPTY_INDEX _("Untitled in index")
 
 
-bool CHMFile::BinaryTOC(wxTreeCtrl *toBuild, const wxCSConv& cv)
+bool CHMFile::BinaryTOC(wxTreeCtrl *toBuild)
 {
 	if(!toBuild)
 		return false;
@@ -276,7 +277,7 @@ bool CHMFile::BinaryTOC(wxTreeCtrl *toBuild, const wxCSConv& cv)
 
 	uint32_t off = UINT32ARRAY(topidx.get());
 	RecurseLoadBTOC(topidx, topics, strings, urltbl, urlstr, off, 
-			cv, toBuild, 1);
+			toBuild, 1);
 
 	return true;
 }
@@ -285,7 +286,6 @@ bool CHMFile::BinaryTOC(wxTreeCtrl *toBuild, const wxCSConv& cv)
 void CHMFile::RecurseLoadBTOC(UCharPtr& topidx, UCharPtr& topics,
 			      UCharPtr& strings, UCharPtr& urltbl,
 			      UCharPtr& urlstr, uint32_t offset,
-			      const wxCSConv& cv, 
 			      wxTreeCtrl *toBuild, int level)
 {
 	while(offset) {
@@ -312,7 +312,7 @@ void CHMFile::RecurseLoadBTOC(UCharPtr& topidx, UCharPtr& topics,
 			if(child) {
 				RecurseLoadBTOC(topidx, topics, strings, 
 						urltbl, urlstr, child,
-						cv, toBuild, level + 1); 
+						toBuild, level + 1); 
 			}
 		}
 
@@ -387,7 +387,8 @@ bool CHMFile::GetItem(UCharPtr& topics, UCharPtr& strings, UCharPtr& urltbl,
 
 	if(tree && !name.empty()) {
 		int parentIndex = level ? level - 1 : 0;
-		tname = CURRENT_CHAR_STRING(name.c_str());
+		tname = translateEncoding(CURRENT_CHAR_STRING(name.c_str()),
+			_enc);
 
 		parents[level] = 
 			tree->AppendItem(parents[parentIndex], tname, 2, 2,
@@ -430,10 +431,8 @@ bool CHMFile::GetTopicsTree(wxTreeCtrl *toBuild)
 	if(!toBuild) 
 		return false;
 
-	wxCSConv cv(_enc);
-	
 	toBuild->Freeze();
-	bool btoc = BinaryTOC(toBuild, cv);
+	bool btoc = BinaryTOC(toBuild);
 	toBuild->Thaw();
 
 	if(btoc)
@@ -641,10 +640,11 @@ bool CHMFile::GetIndex(CHMListCtrl* toBuild)
 	if(!toBuild)
 		return false;
 
-	wxCSConv cv(_enc);
-	
+	std::auto_ptr<wxCSConv> cvPtr;
+	createCSConvPtr(cvPtr, _enc);
+
 	toBuild->Freeze();
-	bool bindex = BinaryIndex(toBuild, cv);
+	bool bindex = BinaryIndex(toBuild, *cvPtr);
 	toBuild->Thaw();
 
 	if(bindex)
@@ -927,8 +927,10 @@ size_t CHMFile::RetrieveObject(chmUnitInfo *ui, unsigned char *buffer,
 
 bool CHMFile::GetArchiveInfo()
 {
-	bool retw = InfoFromWindows();
+	// Order counts, parse #SYSTEM before #WINDOWS
+	// (thanks to Kuang-che Wu for pointing that out)
 	bool rets = InfoFromSystem();
+	bool retw = InfoFromWindows();
 
 	return (retw || rets);
 }
@@ -1057,8 +1059,9 @@ bool CHMFile::ProcessWLC(uint64_t wlc_count, uint64_t wlc_size,
 				topic = CURRENT_CHAR_STRING(combuf);
 #if wxUSE_UNICODE
 			else {
-				wxCSConv cv(_enc);
-				topic = wxString((const char *)combuf, cv);
+				std::auto_ptr<wxCSConv> cvPtr;
+				createCSConvPtr(cvPtr, _enc);
+				topic = wxString((const char *)combuf, *cvPtr);
 			}
 #endif
 		}
@@ -1337,7 +1340,6 @@ bool CHMFile::InfoFromSystem()
 	return true;
 }
 
-
 inline
 wxFontEncoding CHMFile::GetFontEncFromCharSet(int cs)
 {
@@ -1357,7 +1359,7 @@ wxFontEncoding CHMFile::GetFontEncFromCharSet(int cs)
 		fontEncoding = wxFONTENCODING_CP1251;
 		break;
         case ARABIC_CHARSET:
-		fontEncoding = wxFONTENCODING_ISO8859_6;
+		fontEncoding = wxFONTENCODING_CP1256; 
 		break;
         case GREEK_CHARSET:
 		fontEncoding = wxFONTENCODING_ISO8859_7;
@@ -1404,16 +1406,43 @@ wxFontEncoding CHMFile::GetFontEncFromLCID(uint32_t lcid)
 	
 	switch(lid) {
 	case LANG_ARABIC:
+	case LANG_FARSI:
+	case LANG_URDU:
 		fontEncoding = wxFONTENCODING_CP1256; 
+		//fontEncoding = wxFONTENCODING_UTF8; 
 		break;
+	case LANG_AZERI:
+	case LANG_BELARUSIAN:
+	case LANG_BULGARIAN:
+	case LANG_KAZAK:
+	case LANG_KYRGYZ:
+	case LANG_MONGOLIAN:
+	case LANG_TATAR:
 	case LANG_RUSSIAN:
+	case LANG_UKRAINIAN:
+	case LANG_UZBEK:
 		fontEncoding = wxFONTENCODING_CP1251;
 		break;
-	case LANG_UKRAINIAN:
-		fontEncoding = wxFONTENCODING_KOI8_U;
-		break;
-	case LANG_BULGARIAN:
-		fontEncoding = wxFONTENCODING_BULGARIAN;
+	case LANG_AFRIKAANS:
+	case LANG_BASQUE:
+	case LANG_CATALAN:
+	case LANG_DANISH:
+	case LANG_DUTCH:
+	case LANG_ENGLISH:
+	case LANG_FINNISH:
+	case LANG_FRENCH:
+	case LANG_GALICIAN:
+	case LANG_GERMAN:
+	case LANG_ICELANDIC:
+	case LANG_INDONESIAN:
+	case LANG_ITALIAN:
+	case LANG_MALAY:
+	case LANG_NORWEGIAN:
+	case LANG_PORTUGUESE:
+	case LANG_SPANISH:
+	case LANG_SWAHILI:
+	case LANG_SWEDISH:
+		fontEncoding = wxFONTENCODING_CP1252;
 		break;
 	case LANG_GREEK:
 		fontEncoding = wxFONTENCODING_CP1253;
@@ -1439,6 +1468,16 @@ wxFontEncoding CHMFile::GetFontEncFromLCID(uint32_t lcid)
         case LANG_JAPANESE:
 		fontEncoding = wxFONTENCODING_CP932;
 		break;
+	case LANG_ALBANIAN:
+	case LANG_CROATIAN:
+	case LANG_CZECH:
+	case LANG_HUNGARIAN:
+	case LANG_POLISH:
+	case LANG_ROMANIAN:
+	case LANG_SLOVAK:
+	case LANG_SLOVENIAN:
+		fontEncoding = wxFONTENCODING_CP1250;
+		break;
 	case LANG_NEUTRAL:
 	default:
 		fontEncoding = wxFONTENCODING_SYSTEM;
